import java.math.BigDecimal;

// matrix structure
public class Matrix
{
   public int      r;     // rows
   public int      c;     // columns
   public double[] e;     // elements

   public Matrix()
   {
   }


   //allocate matrix (m x n)
   public static Matrix matrix_alloc(int m, int n)
   {
      Matrix mx = new Matrix();

      mx.r = m;
      mx.c = n;
      int j = m * n;
      mx.e = new double[j];
      for (int i = 0; i < j; i++)
      {
         mx.e[i] = 0.0;
      }
      return(mx);
   }


   //allocate identity matrix (n)
   public static Matrix matrix_identity_alloc(int n)
   {
      Matrix mx = matrix_alloc(n, n);
      int    i;

      for (i = 0; i < n; i++)
      {
         mx.e[i * n + i] = 1.0;
      }
      return(mx);
   }


   //allocate diagonal matrix
   public static Matrix matrix_diag_alloc(Matrix A)
   {
      Matrix mx = matrix_alloc(A.r, A.r);
      int    i, n;

      n = A.r;
      for (i = 0; i < n; i++)
      {
         mx.e[i * n + i] = A.e[i];
      }
      return(mx);
   }


   //cross matrix
   public static void matrix3_cross(Matrix A, Matrix B)
   {
      B.e[0] = 0.0f;
      B.e[1] = -A.e[2];
      B.e[2] = A.e[1];
      B.e[3] = A.e[2];
      B.e[5] = -A.e[0];
      B.e[6] = -A.e[1];
      B.e[7] = A.e[0];
      B.e[8] = 0.0f;
   }


   //transpose matrix A (m x n)  to  B (n x m)
   public static void matrix_transpose(Matrix A, Matrix B)
   {
      int i, j;
      int m = A.r;
      int n = A.c;

      B.r = A.c;
      B.c = A.r;
      for (i = 0; i < m; i++)
      {
         for (j = 0; j < n; j++)
         {
            B.e[j * m + i] = A.e[i * n + j];
         }
      }
   }


   //multiply matrix A (m x p) by  B(p x n) , put result in C (m x n)
   public static void matrix_multiply(Matrix A, Matrix B, Matrix C)
   {
      int i, j, k;
      int m = A.r;
      int n = B.c;
      int p = A.c;

      C.r = m;
      C.c = n;
      for (i = 0; i < m; i++)                   //each row in A
      {
         for (j = 0; j < n; j++)                //each column in B
         {
            C.e[i * n + j] = 0;
            for (k = 0; k < p; k++)              //each element in row A & column B
            {
               C.e[i * n + j] += A.e[i * p + k] * B.e[k * n + j];
            }
         }
      }
   }


   //invert square matrix A (m x m), put result in B (m x m)
   // Note: max matrix order is 6.
   public static void matrix_invert(Matrix A, Matrix B)
   {
      double[][] matrix = new double[12][12];
      double ratio, a;
      int    i, j, k, n;

      n = A.r;
      for (i = 0; i < n; i++)
      {
         for (j = 0; j < n; j++)
         {
            matrix[i][j] = A.e[(i * n) + j];
         }
      }
      for (i = 0; i < n; i++)
      {
         for (j = n; j < 2 * n; j++)
         {
            if (i == (j - n))
            {
               matrix[i][j] = 1.0;
            }
            else
            {
               matrix[i][j] = 0.0;
            }
         }
      }
      for (i = 0; i < n; i++)
      {
         for (j = 0; j < n; j++)
         {
            if (i != j)
            {
               ratio = matrix[j][i] / matrix[i][i];
               for (k = 0; k < 2 * n; k++)
               {
                  matrix[j][k] -= ratio * matrix[i][k];
               }
            }
         }
      }
      for (i = 0; i < n; i++)
      {
         a = matrix[i][i];
         for (j = 0; j < 2 * n; j++)
         {
            matrix[i][j] /= a;
         }
      }
      for (i = 0; i < n; i++)
      {
         for (j = n; j < 2 * n; j++)
         {
            B.e[(i * n) + (j - n)] = matrix[i][j];
         }
      }
   }


   //add matrix A (m x n) to  B(m x n) , put result in C (m x n)
   public static void matrix_add(Matrix A, Matrix B, Matrix C)
   {
      int i;
      int j = A.r * A.c;

      for (i = 0; i < j; i++)
      {
         C.e[i] = A.e[i] + B.e[i];
      }
   }


   //subtract matrix B(m x n) from  A(m x n) , put result in C (m x n)
   public static void matrix_subtract(Matrix A, Matrix B, Matrix C)
   {
      int i;
      //assert(A.r == B.r && A.c == B.c);
      //assert(A.r == C.r && A.c == C.c);
      int j = A.r * A.c;

      for (i = 0; i < j; i++)
      {
         C.e[i] = A.e[i] - B.e[i];
      }
   }


   //concatenate matrix A (m x n) and B(m x n) , put result in C (2m x n)
   public static void matrix_concat(Matrix A, Matrix B, Matrix C)
   {
      int i;
      int n = A.r * A.c;

      for (i = 0; i < n; i++)
      {
         C.e[i]     = A.e[i];
         C.e[i + n] = B.e[i];
      }
   }


   //set diagnonal value
   public static void matrix_set_diag(double value, Matrix A)
   {
      int i;
      int n = A.r * A.c;

      for (i = 0; i < n; i++)
      {
         A.e[i] = 0.0;
      }
      n = A.c;
      for (i = 0; i < n; i++)
      {
         A.e[i * n + i] = value;
      }
   }


   //copy matrix A to B
   public static void matrix_copy(Matrix A, Matrix B)
   {
      int i, j;
      //assert(A.r == B.r && A.c == B.c);
      int m = A.r;
      int n = A.c;

      B.r = m;
      B.c = n;
      for (i = 0; i < m; i++)
      {
         for (j = 0; j < n; j++)
         {
            B.e[i * n + j] = A.e[i * n + j];
         }
      }
   }


   /*******************************************************************************
   *  Singular value decomposition program, svdcmp, from "Numerical Recipes in C"
   *  (Cambridge Univ. Press) by W.H. Press, S.A. Teukolsky, W.T. Vetterling,
   *  and B.P. Flannery
   *******************************************************************************/
   public static final int NR_END = 1;
   //#define FREE_ARG char*
   public static double SIGN(double a, double b) { if (b >= 0.0) { return(Math.abs(a)); } else{ return(-Math.abs(a)); } }
   public static double dmaxarg1, dmaxarg2;
   public static double DMAX(double a, double b) { dmaxarg1 = a;
                                                   dmaxarg2 = b;
                                                   if (dmaxarg1 > dmaxarg2) { return(dmaxarg1); } else{ return(dmaxarg2); } }
   public static int iminarg1, iminarg2;
   public static int IMIN(int a, int b) { iminarg1 = a;
                                          iminarg2 = b;
                                          if (iminarg1 < iminarg2) { return(iminarg1); } else{ return(iminarg2); } }

   public static double[] dvector(int nl, int nh)
   /* allocate a double vector with subscript range v[nl..nh] */
   {
      double[] v;
      v = new double[nl + (nh - nl + 1 + NR_END)];
      return(v);
   }


   public static double pythag(double a, double b)
   /* compute (a2 + b2)^1/2 without destructive underflow or overflow */
   {
      double absa, absb;

      absa = Math.abs(a);
      absb = Math.abs(b);
      if (absa > absb) { return(absa * Math.sqrt(1.0 + (absb / absa) * (absb / absa))); }
      else{ return(absb == 0.0 ? 0.0 : absb * Math.sqrt(1.0 + (absa / absb) * (absa / absb))); }
   }


   /*******************************************************************************
   *  Given a 3x3 matrix a, this routine computes its singular value
   *  decomposition, A = U.W.VT.  The matrix U replaces a on output.  The diagonal
   *  matrix of singular values W is output as a vector w[3].  The matrix V (not
   *  the transpose VT) is output as v.
   *******************************************************************************/
   public static void matrix3_svd(Matrix A, Vector3d W, Matrix V)
   {
      int    flag, i, its, j, jj, k, l, nm, m, n;
      double anorm, c, f, g, h, s, scale, x, y, z;

      double[] rv1;
      double[][] a = new double[4][4];
      double[] w   = new double[4];
      double[][] v = new double[4][4];
      m            = n = 3;
      l            = nm = 0;
      for (i = 0; i < m; i++)
      {
         for (j = 0; j < n; j++)
         {
            a[i + 1][j + 1] = A.e[i * m + j];
         }
      }

      rv1 = dvector(1, n);
      g   = scale = anorm = 0.0;         /* Householder reduction to bidiagonal form */
      for (i = 1; i <= n; i++)
      {
         l      = i + 1;
         rv1[i] = scale * g;
         g      = s = scale = 0.0;
         if (i <= m)
         {
            for (k = i; k <= m; k++) { scale += Math.abs(a[k][i]); }
            if (scale != 0.0)
            {
               for (k = i; k <= m; k++)
               {
                  a[k][i] /= scale;
                  s       += a[k][i] * a[k][i];
               }
               f       = a[i][i];
               g       = -SIGN(Math.sqrt(s), f);
               h       = f * g - s;
               a[i][i] = f - g;
               for (j = l; j <= n; j++)
               {
                  for (s = 0.0, k = i; k <= m; k++) { s += a[k][i] * a[k][j]; }
                  f = s / h;
                  for (k = i; k <= m; k++) { a[k][j] += f * a[k][i]; }
               }
               for (k = i; k <= m; k++) { a[k][i] *= scale; }
            }
         }
         w[i] = scale * g;
         g    = s = scale = 0.0;
         if ((i <= m) && (i != n))
         {
            for (k = l; k <= n; k++) { scale += Math.abs(a[i][k]); }
            if (scale != 0.0)
            {
               for (k = l; k <= n; k++)
               {
                  a[i][k] /= scale;
                  s       += a[i][k] * a[i][k];
               }
               f       = a[i][l];
               g       = -SIGN(Math.sqrt(s), f);
               h       = f * g - s;
               a[i][l] = f - g;
               for (k = l; k <= n; k++) { rv1[k] = a[i][k] / h; }
               for (j = l; j <= m; j++)
               {
                  for (s = 0.0, k = l; k <= n; k++) { s += a[j][k] * a[i][k]; }
                  for (k = l; k <= n; k++) { a[j][k] += s * rv1[k]; }
               }
               for (k = l; k <= n; k++) { a[i][k] *= scale; }
            }
         }
         anorm = DMAX(anorm, (Math.abs(w[i]) + Math.abs(rv1[i])));
      }
      for (i = n; i >= 1; i--)           /* Accumulation of right-hand transformations. */
      {
         if (i < n)
         {
            if (g != 0.0)
            {
               for (j = l; j <= n; j++)                        /* Double division to avoid possible underflow. */
               {
                  v[j][i] = (a[i][j] / a[i][l]) / g;
               }
               for (j = l; j <= n; j++)
               {
                  for (s = 0.0, k = l; k <= n; k++) { s += a[i][k] * v[k][j]; }
                  for (k = l; k <= n; k++) { v[k][j] += s * v[k][i]; }
               }
            }
            for (j = l; j <= n; j++) { v[i][j] = v[j][i] = 0.0; }
         }
         v[i][i] = 1.0;
         g       = rv1[i];
         l       = i;
      }
      for (i = IMIN(m, n); i >= 1; i--)           /* Accumulation of left-hand transformations. */
      {
         l = i + 1;
         g = w[i];
         for (j = l; j <= n; j++) { a[i][j] = 0.0; }
         if (g != 0.0)
         {
            g = 1.0 / g;
            for (j = l; j <= n; j++)
            {
               for (s = 0.0, k = l; k <= m; k++) { s += a[k][i] * a[k][j]; }
               f = (s / a[i][i]) * g;
               for (k = i; k <= m; k++) { a[k][j] += f * a[k][i]; }
            }
            for (j = i; j <= m; j++) { a[j][i] *= g; }
         }
         else{ for (j = i; j <= m; j++) { a[j][i] = 0.0; } }
         ++a[i][i];
      }
      for (k = n; k >= 1; k--)           /* Diagonalization of the bidiagonal form. */
      {
         for (its = 1; its <= 30; its++)
         {
            flag = 1;
            for (l = k; l >= 1; l--)                /* Test for splitting. */
            {
               nm = l - 1;                          /* Note that rv1[1] is always zero. */
               if ((double)(Math.abs(rv1[l]) + anorm) == anorm)
               {
                  flag = 0;
                  break;
               }
               if ((double)(Math.abs(w[nm]) + anorm) == anorm) { break; }
            }
            if (flag != 0)
            {
               c = 0.0;                          /* Cancellation of rv1[l], if l > 1. */
               s = 1.0;
               for (i = l; i <= k; i++)
               {
                  f      = s * rv1[i];
                  rv1[i] = c * rv1[i];
                  if ((double)(Math.abs(f) + anorm) == anorm) { break; }
                  g    = w[i];
                  h    = pythag(f, g);
                  w[i] = h;
                  h    = 1.0 / h;
                  c    = g * h;
                  s    = -f * h;
                  for (j = 1; j <= m; j++)
                  {
                     y        = a[j][nm];
                     z        = a[j][i];
                     a[j][nm] = y * c + z * s;
                     a[j][i]  = z * c - y * s;
                  }
               }
            }
            z = w[k];
            if (l == k)                       /* Convergence. */
            {
               if (z < 0.0)                   /* Singular value is made nonnegative. */
               {
                  w[k] = -z;
                  for (j = 1; j <= n; j++) { v[j][k] = -v[j][k]; }
               }
               break;
            }
            if (its == 30) { System.out.println("no convergence in 30 svdcmp iterations"); }
            x  = w[l];                    /* Shift from bottom 2-by-2 minor. */
            nm = k - 1;
            y  = w[nm];
            g  = rv1[nm];
            h  = rv1[k];
            f  = ((y - z) * (y + z) + (g - h) * (g + h)) / (2.0 * h * y);
            g  = pythag(f, 1.0);
            f  = ((x - z) * (x + z) + h * ((y / (f + SIGN(g, f))) - h)) / x;
            c  = s = 1.0;                    /* Next QR transformation: */
            for (j = l; j <= nm; j++)
            {
               i      = j + 1;
               g      = rv1[i];
               y      = w[i];
               h      = s * g;
               g      = c * g;
               z      = pythag(f, h);
               rv1[j] = z;
               c      = f / z;
               s      = h / z;
               f      = x * c + g * s;
               g      = g * c - x * s;
               h      = y * s;
               y     *= c;
               for (jj = 1; jj <= n; jj++)
               {
                  x        = v[jj][j];
                  z        = v[jj][i];
                  v[jj][j] = x * c + z * s;
                  v[jj][i] = z * c - x * s;
               }
               z    = pythag(f, h);
               w[j] = z;                          /* Rotation can be arbitrary if z = 0. */
               if (z != 0.0)
               {
                  z = 1.0 / z;
                  c = f * z;
                  s = h * z;
               }
               f = c * g + s * y;
               x = c * y - s * g;
               for (jj = 1; jj <= m; jj++)
               {
                  y        = a[jj][j];
                  z        = a[jj][i];
                  a[jj][j] = y * c + z * s;
                  a[jj][i] = z * c - y * s;
               }
            }
            rv1[l] = 0.0;
            rv1[k] = f;
            w[k]   = x;
         }
      }
      for (i = 1; i <= n; i++)
      {
         f       = a[i][2];
         a[i][2] = a[i][3];
         a[i][3] = f;
         f       = v[i][2];
         v[i][2] = v[i][3];
         v[i][3] = f;
      }
      f    = w[2];
      w[2] = w[3];
      w[3] = f;
      for (i = 1; i <= m; i++)
      {
         for (j = 1; j <= n; j++)
         {
            A.e[((i - 1) * m) + (j - 1)] = a[i][j];
            V.e[((i - 1) * m) + (j - 1)] = v[i][j];
         }
      }
      W.x = w[1];
      W.y = w[2];
      W.z = w[3];
   }


   /******************************************************************************
    * matrix_pinv computes the pseudoinverse of a 3x3 matrix.
    *
    * Given the singular value decomposition of a real MxN matrix A:
    *
    * A = U * S * V'
    *
    * where
    *
    * U is MxM orthogonal,
    * S is MxN, and entirely zero except for the diagonal;
    * V is NxN orthogonal.
    *
    * the pseudo inverse is the NxM matrix A+ with the form
    *
    * A+ = V * S+ * U'
    *
    * where
    *
    * S+ is the NxM matrix whose nonzero diagonal elements are
    * the inverses of the corresponding diagonal elements of S.
    */
   public static void matrix3_pinv(Matrix A, Matrix Ai)
   {
      Matrix   Ait, U, S, V, Vt, T;
      Vector3d W;

      // SVD of A.
      U = matrix_alloc(3, 3);
      V = matrix_alloc(3, 3);
      W = Vector3d.vector3d_alloc();
      matrix_copy(A, U);
      matrix3_svd(U, W, V);

      // Pseudo inverse.
      Ait = matrix_alloc(3, 3);
      S   = matrix_alloc(3, 3);
      Vt  = matrix_alloc(3, 3);
      T   = matrix_alloc(3, 3);
      if (W.x != 0.0)
      {
         S.e[0] = 1.0 / W.x;
      }
      if (W.y != 0.0)
      {
         S.e[4] = 1.0 / W.y;
      }
      if (W.z != 0.0)
      {
         S.e[8] = 1.0 / W.z;
      }
      matrix_transpose(V, Vt);
      matrix_multiply(U, S, T);
      matrix_multiply(T, Vt, Ait);
      matrix_transpose(Ait, Ai);
   }


   //inverse of 3x3 matrix
   //inverse of matrix A (3 x 3), put result in B (3 x 3)
   public static void matrix3_inv(Matrix A, Matrix B)
   {
      double[][] m    = new double[3][3];
      double[][] minv = new double[3][3];

      double det = new LUDecomposition(A).det();
      if (det == 0.0) { return; }
      double invdet = 1.0 / det;

      m[0][0] = A.e[0];
      m[0][1] = A.e[1];
      m[0][2] = A.e[2];
      m[1][0] = A.e[3];
      m[1][1] = A.e[4];
      m[1][2] = A.e[5];
      m[2][0] = A.e[6];
      m[2][1] = A.e[7];
      m[2][2] = A.e[8];

      minv[0][0] = (m[1][1] * m[2][2] - m[2][1] * m[1][2]) * invdet;
      minv[0][1] = (m[0][2] * m[2][1] - m[0][1] * m[2][2]) * invdet;
      minv[0][2] = (m[0][1] * m[1][2] - m[0][2] * m[1][1]) * invdet;
      minv[1][0] = (m[1][2] * m[2][0] - m[1][0] * m[2][2]) * invdet;
      minv[1][1] = (m[0][0] * m[2][2] - m[0][2] * m[2][0]) * invdet;
      minv[1][2] = (m[1][0] * m[0][2] - m[0][0] * m[1][2]) * invdet;
      minv[2][0] = (m[1][0] * m[2][1] - m[2][0] * m[1][1]) * invdet;
      minv[2][1] = (m[2][0] * m[0][1] - m[0][0] * m[2][1]) * invdet;
      minv[2][2] = (m[0][0] * m[1][1] - m[1][0] * m[0][1]) * invdet;

      B.e[0] = minv[0][0];
      B.e[1] = minv[0][1];
      B.e[2] = minv[0][2];
      B.e[3] = minv[1][0];
      B.e[4] = minv[1][1];
      B.e[5] = minv[1][2];
      B.e[6] = minv[2][0];
      B.e[7] = minv[2][1];
      B.e[8] = minv[2][2];
   }


   /**
    * returns the square root of the matrix, i.e., X from the equation X*X = A.<br/>
    * Steps in the Calculation (see <a href=
    * "http://www.mathworks.com/access/helpdesk/help/techdoc/ref/sqrtm.html"
    * target="blank"><code>sqrtm</code></a> in Matlab):<br/>
    * <ol>
    * <li>perform eigenvalue decomposition<br/>
    * [V,D]=eig(A)</li>
    * <li>take the square root of all elements in D (only the ones with positive
    * sign are considered for further computation)<br/>
    * S=sqrt(D)</li>
    * <li>calculate the root<br/>
    * X=V*S/V, which can be also written as X=(V'\(V*S)')'</li>
    * </ol>
    * <p/>
    * <b>Note:</b> since this method uses other high-level methods, it generates
    * several instances of matrices. This can be problematic with large matrices.
    * <p/>
    * Examples:
    * <ol>
    * <li>
    *
    * <pre>
    *  X =
    *   5   -4    1    0    0
    *  -4    6   -4    1    0
    *   1   -4    6   -4    1
    *   0    1   -4    6   -4
    *   0    0    1   -4    5
    *
    *  sqrt(X) =
    *   2   -1   -0   -0   -0
    *  -1    2   -1    0   -0
    *   0   -1    2   -1    0
    *  -0    0   -1    2   -1
    *  -0   -0   -0   -1    2
    *
    *  Matrix m = new Matrix(new double[][]{{5,-4,1,0,0},{-4,6,-4,1,0},{1,-4,6,-4,1},{0,1,-4,6,-4},{0,0,1,-4,5}});
    * </pre>
    *
    * </li>
    * <li>
    *
    * <pre>
    *  X =
    *   7   10
    *  15   22
    *
    *  sqrt(X) =
    *  1.5667    1.7408
    *  2.6112    4.1779
    *
    *  Matrix m = new Matrix(new double[][]{{7, 10},{15, 22}});
    * </pre>
    *
    * </li>
    * </ol>
    *
    * @return sqrt(A)
    */

   //square root of 3x3 matrix
   public static void matrix3_sqrt(Matrix A, Matrix B)
   {
      EigenvalueDecomposition evd;
      Matrix s;
      Matrix v;
      Matrix d;
      Matrix a;
      Matrix b;
      int    i;
      int    n;

      // eigenvalue decomp.
      // [V, D] = eig(A) with A = this
      evd = new EigenvalueDecomposition(A);
      v   = evd.getV();
      d   = evd.getD();

      // S = sqrt of cells of D
      s = Matrix.matrix_alloc(d.r, d.c);
      for (i = 0, n = s.r * s.c; i < n; i++)
      {
         s.e[i] = StrictMath.sqrt(d.e[i]);
      }

      // to calculate:
      // result = V*S/V
      //
      // with X = B/A
      // and B/A = (A'\B')'
      // and V=A and V*S=B
      // we get
      // result = (V'\(V*S)')'
      //
      // A*X = B
      // X = A\B
      // which is
      // X = A.solve(B)
      //
      // with A=V' and B=(V*S)'
      // we get
      // X = V'.solve((V*S)')
      // or
      // result = X'
      //
      // which is in full length
      // result = (V'.solve((V*S)'))'
      a = Matrix.matrix_alloc(v.r, v.c);
      Matrix.matrix3_inv(v, a);
      Matrix t = Matrix.matrix_alloc(v.r, v.c);
      Matrix.matrix_multiply(v, s, t);
      b = Matrix.matrix_alloc(v.r, v.c);
      Matrix.matrix3_inv(t, b);
      v   = null;
      d   = null;
      evd = null;
      s   = null;
      LUDecomposition lud = new LUDecomposition(a);
      s = lud.solve(b);
      Matrix.matrix3_inv(s, B);
   }


   //square root of 3x3 matrix
   // Denman-Beavers method.
   public static void matrix3_sqrt_old(Matrix A, Matrix B)
   {
      int    i, n;
      Matrix Ad;
      Matrix Bd;

      Ad = matrix_alloc(3, 3);
      Bd = matrix_alloc(3, 3);
      for (i = 0; i < 9; i++)
      {
         Ad.e[i] = A.e[i];
      }

      Matrix Y = matrix_alloc(3, 3);
      matrix_copy(Ad, Y);
      Matrix Yi = matrix_alloc(3, 3);
      Matrix Yn = matrix_alloc(3, 3);
      Matrix Z  = matrix_identity_alloc(3);
      Matrix Zi = matrix_alloc(3, 3);
      Matrix Zn = matrix_alloc(3, 3);
      for (n = 0; n < 20; n++)
      {
         matrix3_pinv(Z, Zi);
         matrix_add(Y, Zi, Yn);
         for (i = 0; i < 9; i++)
         {
            Yn.e[i] *= 0.5;
         }
         matrix3_pinv(Y, Yi);
         matrix_add(Z, Yi, Zn);
         for (i = 0; i < 9; i++)
         {
            Zn.e[i] *= 0.5;
         }
         matrix_copy(Yn, Y);
         matrix_copy(Zn, Z);
      }
      matrix_copy(Y, Bd);
      for (i = 0; i < 9; i++)
      {
         B.e[i] = Bd.e[i];
      }
   }


   /**
    * Get a submatrix.
    *
    * @param ar Array of row indices.
    * @param j0 Initial column index
    * @param j1 Final column index
    * @return A(r(:),j0:j1)
    * @throws ArrayIndexOutOfBoundsException Submatrix indices
    */
   public Matrix getMatrix(int[] ar, int j0, int j1)
   {
      int    xr = ar.length;
      int    xc = j1 - j0 + 1;
      Matrix X  = Matrix.matrix_alloc(xr, xc);

      double[][] B = new double[xr][xc];
      try {
         for (int i = 0; i < ar.length; i++)
         {
            for (int j = j0; j <= j1; j++)
            {
               B[i][j - j0] = e[(ar[i] * r) + j];
            }
         }
      }
      catch (ArrayIndexOutOfBoundsException e) {
         throw new ArrayIndexOutOfBoundsException("Submatrix indices");
      }
      for (int i = 0; i < xr; i++)
      {
         for (int j = 0; j < xc; j++)
         {
            X.e[(i * xr) + j] = B[i][j];
         }
      }
      return(X);
   }


   //print matrix
   public static void matrix_print(Matrix A)
   {
      int i, j;
      int m = A.r;
      int n = A.c;

      for (i = 0; i < m; i++)
      {
         System.out.print("{");
         for (j = 0; j < n; j++)
         {
            if (j > 0)
            {
               System.out.print(",");
            }
            System.out.print(A.e[i * n + j] + "");
         }
         if (i < m - 1) { System.out.println("},"); } else{ System.out.println("}"); }
      }
   }
}
